import { getSchemaProperties } from "../schema/mod.js";
import { CONSTRUCT, DELETE, INSERT, SELECT, sparql as $, } from "../sparql/mod.js";
import { DataFactory } from "../rdf.js";
import ldkit from "../namespaces/ldkit.js";
import rdf from "../namespaces/rdf.js";
import { encode } from "../encoder.js";
import { QueryHelper } from "./query_helper.js";
export class QueryBuilder {
    constructor(schema, context) {
        Object.defineProperty(this, "schema", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "schemaProperties", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "context", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "df", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "deleteQuery", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (iris) => {
                return DELETE `
      ?s ?p ?o
    `.WHERE `
      ?s ?p ?o .
      VALUES ?s { ${iris.map(this.df.namedNode)} }
    `.build();
            }
        });
        this.schema = schema;
        this.schemaProperties = getSchemaProperties(this.schema);
        this.context = context;
        this.df = new DataFactory();
    }
    getResourceSignature() {
        return this.df.quad(this.df.variable("iri"), this.df.namedNode(rdf.type), this.df.namedNode(ldkit.Resource));
    }
    getTypesSignature() {
        return this.df.quad(this.df.variable("iri"), this.df.namedNode(rdf.type), this.df.variable("iri_type"));
    }
    entitiesToQuads(entities) {
        const quadArrays = entities.map((entity) => encode(entity, this.schema, this.context));
        return [].concat(...quadArrays);
    }
    getShape(includeOptional = false, wrapOptional = true, omitRootTypes = false) {
        const mainVar = "iri";
        const conditions = [];
        const populateConditionsRecursive = (s, varPrefix) => {
            const rdfType = s["@type"];
            const properties = getSchemaProperties(s);
            if (varPrefix !== "iri" || !omitRootTypes) {
                rdfType.forEach((type) => {
                    conditions.push(this.df.quad(this.df.variable(varPrefix), this.df.namedNode(rdf.type), this.df.namedNode(type)));
                });
            }
            Object.keys(properties).forEach((prop, index) => {
                const property = properties[prop];
                const isOptional = property["@optional"];
                if (!includeOptional && isOptional) {
                    return;
                }
                if (wrapOptional && isOptional) {
                    conditions.push($ `\nOPTIONAL {`);
                }
                conditions.push(this.df.quad(this.df.variable(varPrefix), this.df.namedNode(property["@id"]), this.df.variable(`${varPrefix}_${index}`)));
                if (typeof property["@context"] === "object") {
                    populateConditionsRecursive(property["@context"], `${varPrefix}_${index}`);
                }
                if (wrapOptional && isOptional) {
                    conditions.push($ `\n}\n`);
                }
            });
        };
        populateConditionsRecursive(this.schema, mainVar);
        return conditions;
    }
    countQuery() {
        const quads = this.getShape();
        return SELECT `(count(?iri) as ?count)`.WHERE `${quads}`.build();
    }
    getQuery(where, limit = 1000) {
        const selectSubQuery = SELECT `
      ${this.df.variable("iri")}
    `.WHERE `
      ${this.getShape(false, true)}
      ${where}
    `.LIMIT(limit).build();
        const query = CONSTRUCT `
      ${this.getResourceSignature()}
      ${this.getTypesSignature()}
      ${this.getShape(true, false, true)}
    `.WHERE `
      ${this.getTypesSignature()}
      ${this.getShape(true, true, true)}
      {
        ${selectSubQuery}
      }
    `.build();
        return query;
    }
    getByIrisQuery(iris) {
        const query = CONSTRUCT `
      ${this.getResourceSignature()}
      ${this.getTypesSignature()}
      ${this.getShape(true, false, true)}
    `.WHERE `
      ${this.getTypesSignature()}
      ${this.getShape(true, true, true)}
      VALUES ?iri {
        ${iris.map(this.df.namedNode)}
      }
    `.build();
        return query;
    }
    insertQuery(entities) {
        const quads = this.entitiesToQuads(entities);
        return this.insertDataQuery(quads);
    }
    insertDataQuery(quads) {
        return INSERT.DATA `${quads}`.build();
    }
    deleteDataQuery(quads) {
        return DELETE.DATA `${quads}`.build();
    }
    updateQuery(entities) {
        const deleteQuads = [];
        const insertQuads = [];
        const whereQuads = [];
        entities.forEach((entity, index) => {
            const helper = new QueryHelper(entity, this.schema, this.context, 1000 * index);
            deleteQuads.push(...helper.getDeleteQuads());
            insertQuads.push(...helper.getInsertQuads());
            whereQuads.push(...helper.getWhereQuads());
        });
        return DELETE `${deleteQuads}`.INSERT `${insertQuads}`
            .WHERE `${deleteQuads}`.build();
    }
}
