"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = exports.getResponseTypes = void 0;
const rdf_js_1 = require("../rdf.js");
const asynciterator_js_1 = require("../asynciterator.js");
class QueryResolver {
}
class BooleanJsonResolver extends QueryResolver {
    async resolve(response) {
        const json = await response.json();
        if ("boolean" in json) {
            return Boolean(json.boolean);
        }
        throw new Error("Boolean SPARQL query result not found");
    }
}
class BindingsJsonResolver extends QueryResolver {
    async resolve(response) {
        const json = await response.json();
        if (!Array.isArray(json.results?.bindings)) {
            throw new Error("Bindings SPARQL query result not found");
        }
        const bindingsFactory = new rdf_js_1.BindingsFactory();
        const bindingsIterator = new asynciterator_js_1.ArrayIterator(json.results.bindings);
        // TODO: review the unknown type cast
        return new asynciterator_js_1.MappingIterator(bindingsIterator, (i) => bindingsFactory.fromJson(i));
    }
}
class QuadsJsonResolver extends QueryResolver {
    async resolve(response) {
        const json = await response.json();
        if (!(json?.constructor === Object)) {
            throw new Error("Quads SPARQL query result not found");
        }
        const quadFactory = new rdf_js_1.QuadFactory();
        const treeIterator = new asynciterator_js_1.TreeIterator(json);
        // TODO: review the unknown type cast
        return new asynciterator_js_1.MappingIterator(treeIterator, (i) => quadFactory.fromJson(i));
    }
}
class QuadsTurtleResolver extends QueryResolver {
    async resolve(response) {
        const text = await response.text();
        const quads = new rdf_js_1.N3.Parser({ format: "turtle" }).parse(text);
        return new asynciterator_js_1.ArrayIterator(quads);
    }
}
const resolvers = {
    "boolean": {
        "application/sparql-results+json": new BooleanJsonResolver(),
    },
    "bindings": {
        "application/sparql-results+json": new BindingsJsonResolver(),
    },
    "quads": {
        "application/rdf+json": new QuadsJsonResolver(),
        "text/turtle": new QuadsTurtleResolver(),
    },
};
const getResponseTypes = (resolverType) => Object.keys(resolvers[resolverType]);
exports.getResponseTypes = getResponseTypes;
const resolve = (resolverType, response) => {
    const contentType = response.headers.get("Content-type");
    if (!contentType) {
        throw new Error(`Content-type header was not found in response`);
    }
    const separatorPosition = contentType.indexOf(";");
    const mime = separatorPosition > 0
        ? contentType.substring(0, separatorPosition)
        : contentType;
    const resolver = resolvers[resolverType][mime];
    if (!resolver) {
        throw new Error(`No resolver exists for response type '${mime}'`);
    }
    return resolver.resolve(response);
};
exports.resolve = resolve;
