"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lens = exports.createResource = exports.createLens = void 0;
const global_js_1 = require("../global.js");
const mod_js_1 = require("../schema/mod.js");
const decoder_js_1 = require("../decoder.js");
const query_builder_js_1 = require("./query_builder.js");
const query_engine_proxy_js_1 = require("../engine/query_engine_proxy.js");
/**
 * Lens lets you query and update RDF data via data schema using TypeScript native data types.
 *
 * https://ldkit.io/docs/components/lens
 *
 * @param schema data schema which extends `SchemaPrototype`
 * @param context optional `Context` - contains LDkit and query engine configuration
 * @param engine optional Query Engine
 * @returns Lens instance
 */
const createLens = (schema, context, engine) => new Lens(schema, context, engine);
exports.createLens = createLens;
/**
 * @deprecated
 * Use `createLens` instead
 */
const createResource = (schema, context, engine) => new Lens(schema, context, engine);
exports.createResource = createResource;
class Lens {
    constructor(schema, context, engine) {
        Object.defineProperty(this, "schema", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "context", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "engine", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "queryBuilder", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.schema = (0, mod_js_1.expandSchema)(schema);
        this.context = (0, global_js_1.resolveContext)(context);
        this.engine = new query_engine_proxy_js_1.QueryEngineProxy(this.context, engine);
        this.queryBuilder = new query_builder_js_1.QueryBuilder(this.schema, this.context);
    }
    decode(graph) {
        return (0, decoder_js_1.decode)(graph, this.schema, this.context);
    }
    async count() {
        const q = this.queryBuilder.countQuery();
        // TODO: console.log(q);
        const bindings = await this.engine.queryBindings(q);
        return parseInt(bindings[0].get("count").value);
    }
    //exists(entity: Identity) {}
    async query(sparqlConstructQuery) {
        const graph = await this.engine.queryGraph(sparqlConstructQuery);
        return this.decode(graph);
    }
    async find(where, limit) {
        const q = this.queryBuilder.getQuery(where, limit);
        // TODO: console.log(q);
        const graph = await this.engine.queryGraph(q);
        return this.decode(graph);
    }
    async findByIri(iri) {
        const results = await this.findByIris([iri]);
        return results.length > 0 ? results[0] : undefined;
    }
    async findByIris(iris) {
        const q = this.queryBuilder.getByIrisQuery(iris);
        // TODO: console.log(q);
        const graph = await this.engine.queryGraph(q);
        return this.decode(graph);
    }
    updateQuery(query) {
        // TODO: console.log(query);
        return this.engine.queryVoid(query);
    }
    insert(...entities) {
        const q = this.queryBuilder.insertQuery(entities);
        return this.updateQuery(q);
    }
    insertData(...quads) {
        const q = this.queryBuilder.insertDataQuery(quads);
        return this.updateQuery(q);
    }
    update(...entities) {
        const q = this.queryBuilder.updateQuery(entities);
        return this.updateQuery(q);
    }
    delete(...identities) {
        const iris = identities.map((identity) => {
            return typeof identity === "string" ? identity : identity.$id;
        });
        const q = this.queryBuilder.deleteQuery(iris);
        return this.updateQuery(q);
    }
    deleteData(...quads) {
        const q = this.queryBuilder.deleteDataQuery(quads);
        return this.updateQuery(q);
    }
}
exports.Lens = Lens;
